<?php

namespace App\Domain\Catalogs\Actions;

use App\Domain\Catalogs\Models\CatalogItem;
use App\Domain\Support\Concerns\AppliesToAggregate;
use App\Domain\Support\Models\Model;
use App\Exceptions\IllegalOperationException;

class ReplaceCatalogItemAction
{
    use AppliesToAggregate;

    public function execute(int $catalogId, array $fields): CatalogItem
    {
        if (isset($fields['parent_id'])) {
            $parentItem = CatalogItem::find($fields['parent_id']);

            throw_if(
                $fields['catalog_id'] !== $parentItem->catalog_id,
                IllegalOperationException::class,
                "The catalog_id {$fields['catalog_id']} must match the parent catalog_id {$parentItem->catalog_id}."
            );
        }

        return $this->replace($catalogId, $fields);
    }

    protected function createModel(): Model
    {
        return new CatalogItem();
    }
}
