<?php

namespace App\Domain\Catalogs\Models;

use App\Domain\Catalogs\Models\Tests\Factories\CatalogsFactory;
use App\Domain\Support\Models\Model;
use Cviebrock\EloquentSluggable\Sluggable;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Support\Collection;

/**
 * Class-model for entity "Catalog".
 *
 * @property string $name
 * @property string $code Catalog unique code
 * @property bool $is_active Catalog is active
 *
 * @property-read Collection|CatalogItem[] $catalogItems
 */
class Catalog extends Model
{
    use Sluggable;

    protected $table = 'catalogs';

    protected $attributes = [
        'is_active' => false,
    ];

    protected $fillable = [
        'name',
        'code',
        'is_active',
    ];

    protected $casts = [
        'is_active' => 'bool',
    ];

    public function catalogItems(): HasMany
    {
        return $this->hasMany(CatalogItem::class, 'catalog_id', 'id');
    }

    public function sluggable(): array
    {
        return [
            'code' => [
                'source' => 'name',
                'separator' => '_',
                'unique' => true,
                'maxLength' => 30,
            ],
        ];
    }

    public static function factory(): CatalogsFactory
    {
        return CatalogsFactory::new();
    }
}
