<?php

namespace App\Domain\Catalogs\Models\Tests\Factories;

use App\Domain\Catalogs\Models\Catalog;
use App\Domain\Catalogs\Models\CatalogItem;
use App\Domain\Categories\Models\Category;
use Ensi\LaravelTestFactories\BaseModelFactory;
use Illuminate\Support\Arr;
use Illuminate\Support\Collection;
use Webmozart\Assert\Assert;

/**
 * @method CatalogItem createOne(array $fields = [])
 * @method CatalogItem makeOne(array $fields = [])
 * @method CatalogItem|CatalogItem[]|Collection<CatalogItem> create(array $fields = [], $parent = null)
 */
class CatalogItemsFactory extends BaseModelFactory
{
    protected $model = CatalogItem::class;

    public function definition(): array
    {
        return [
            'name' => $this->faker->word,
            'url' => $this->faker->nullable()->url,
            'code' => $this->faker->unique()->regexify('[A-Za-z0-9_-]{30}'),
            'catalog_id' => Catalog::factory(),
            'is_active' => $this->faker->boolean(),
        ];
    }

    public function withCatalog(?Catalog $catalog = null): self
    {
        return $this->state(['catalog_id' => $catalog->id ?? Catalog::factory()->create()->id]);
    }

    public function withCategory(?Category $category = null): self
    {
        return $this->state(['category_id' => $category->id ?? Category::factory()->create()->id]);
    }

    public function withParent(?CatalogItem $catalogItem = null): self
    {
        return $this->state(['parent_id' => $catalogItem->id ?? CatalogItem::factory()->create()->id]);
    }

    public function createTree(int $depth = 2): Collection
    {
        Assert::greaterThan($depth, 0);

        $ids = [];

        for ($i = 0; $i < $depth; $i++) {
            $ids[] = $this->createOne(['parent_id' => Arr::last($ids), 'is_active' => true])->id;
        }

        // Load from DB, because parents have fields _lft, _gt changed
        return CatalogItem::query()->whereIn('id', $ids)
            ->orderBy('id')
            ->get();
    }
}
