<?php

namespace App\Http\ApiV1\Modules\Catalogs\Controllers;

use App\Domain\Catalogs\Actions\CreateCatalogItemAction;
use App\Domain\Catalogs\Actions\DeleteCatalogItemAction;
use App\Domain\Catalogs\Actions\PatchCatalogItemAction;
use App\Domain\Catalogs\Actions\ReplaceCatalogItemAction;
use App\Http\ApiV1\Modules\Catalogs\Queries\CatalogItemsQuery;
use App\Http\ApiV1\Modules\Catalogs\Requests\CreateCatalogItemRequest;
use App\Http\ApiV1\Modules\Catalogs\Requests\PatchCatalogItemRequest;
use App\Http\ApiV1\Modules\Catalogs\Requests\ReplaceCatalogItemRequest;
use App\Http\ApiV1\Modules\Catalogs\Resources\CatalogItemsResource;
use App\Http\ApiV1\Support\Pagination\PageBuilderFactory;
use App\Http\ApiV1\Support\Resources\EmptyResource;
use Illuminate\Contracts\Support\Responsable;

class CatalogItemsController
{
    public function create(CreateCatalogItemRequest $request, CreateCatalogItemAction $action): Responsable
    {
        return new CatalogItemsResource($action->execute($request->validated()));
    }

    public function get(int $id, CatalogItemsQuery $query): Responsable
    {
        return new CatalogItemsResource($query->findOrFail($id));
    }

    public function replace(int $id, ReplaceCatalogItemRequest $request, ReplaceCatalogItemAction $action): Responsable
    {
        return new CatalogItemsResource($action->execute($id, $request->validated()));
    }

    public function delete(int $id, DeleteCatalogItemAction $action): Responsable
    {
        $action->execute($id);

        return new EmptyResource();
    }

    public function patch(int $id, PatchCatalogItemRequest $request, PatchCatalogItemAction $action): Responsable
    {
        return new CatalogItemsResource($action->execute($id, $request->validated()));
    }

    public function search(PageBuilderFactory $pageBuilderFactory, CatalogItemsQuery $query): Responsable
    {
        return CatalogItemsResource::collectPage(
            $pageBuilderFactory->fromQuery($query)->build()
        );
    }
}
