<?php

namespace App\Http\ApiV1\Modules\Catalogs\Controllers;

use App\Domain\Catalogs\Actions\CreateCatalogAction;
use App\Domain\Catalogs\Actions\DeleteCatalogAction;
use App\Domain\Catalogs\Actions\PatchCatalogAction;
use App\Domain\Catalogs\Actions\ReplaceCatalogAction;
use App\Http\ApiV1\Modules\Catalogs\Queries\CatalogMultiQuery;
use App\Http\ApiV1\Modules\Catalogs\Queries\CatalogsQuery;
use App\Http\ApiV1\Modules\Catalogs\Requests\CreateCatalogRequest;
use App\Http\ApiV1\Modules\Catalogs\Requests\MultiCatalogRequest;
use App\Http\ApiV1\Modules\Catalogs\Requests\PatchCatalogRequest;
use App\Http\ApiV1\Modules\Catalogs\Requests\ReplaceCatalogRequest;
use App\Http\ApiV1\Modules\Catalogs\Resources\CatalogMultiResourceCollection;
use App\Http\ApiV1\Modules\Catalogs\Resources\CatalogsResource;
use App\Http\ApiV1\Support\Pagination\PageBuilderFactory;
use App\Http\ApiV1\Support\Resources\EmptyResource;
use Illuminate\Contracts\Support\Responsable;

class CatalogsController
{
    public function create(CreateCatalogRequest $request, CreateCatalogAction $action): Responsable
    {
        return new CatalogsResource($action->execute($request->validated()));
    }

    public function get(int $id, CatalogsQuery $query): Responsable
    {
        return new CatalogsResource($query->findOrFail($id));
    }

    public function replace(int $id, ReplaceCatalogRequest $request, ReplaceCatalogAction $action): Responsable
    {
        return new CatalogsResource($action->execute($id, $request->validated()));
    }

    public function delete(int $id, DeleteCatalogAction $action): Responsable
    {
        $action->execute($id);

        return new EmptyResource();
    }

    public function patch(int $id, PatchCatalogRequest $request, PatchCatalogAction $action): Responsable
    {
        return new CatalogsResource($action->execute($id, $request->validated()));
    }

    public function search(PageBuilderFactory $pageBuilderFactory, CatalogsQuery $query): Responsable
    {
        return CatalogsResource::collectPage(
            $pageBuilderFactory->fromQuery($query)->build()
        );
    }

    public function multi(MultiCatalogRequest $request, CatalogMultiQuery $query): Responsable
    {
        return new CatalogMultiResourceCollection($query->get(), $request->input('filter.is_active', false));
    }
}
