<?php

namespace App\Http\ApiV1\Modules\Catalogs\Queries;

use App\Domain\Catalogs\Models\CatalogItem;
use Ensi\QueryBuilderHelpers\Filters\DateFilter;
use Spatie\QueryBuilder\AllowedFilter;
use Spatie\QueryBuilder\QueryBuilder;

class CatalogItemsQuery extends QueryBuilder
{
    public function __construct()
    {
        parent::__construct(CatalogItem::query());

        $this->allowedSorts([
            'id',
            'name',
            'code',
            'url',
            'is_active',
            'created_at',
            'updated_at',
        ]);

        $this->defaultSort('id');

        $this->allowedFilters([
            AllowedFilter::exact('id'),
            AllowedFilter::partial('code'),
            AllowedFilter::partial('name'),
            AllowedFilter::partial('url'),
            AllowedFilter::exact('parent_id'),
            AllowedFilter::exact('catalog_id'),
            AllowedFilter::exact('is_active'),

            ...DateFilter::make('created_at')->gte()->lte(),
            ...DateFilter::make('updated_at')->gte()->lte(),
        ]);

        $this->allowedIncludes([
            'catalog',
            'category',
        ]);
    }
}
