<?php

namespace App\Http\ApiV1\Modules\Catalogs\Queries;

use App\Domain\Catalogs\Models\Catalog;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Spatie\QueryBuilder\AllowedFilter;
use Spatie\QueryBuilder\AllowedInclude;
use Spatie\QueryBuilder\QueryBuilder;

class CatalogMultiQuery extends QueryBuilder
{
    public function __construct()
    {
        parent::__construct(
            Catalog::query()
                ->select(['id', 'name', 'code', 'is_active'])
        );

        $this->allowedSorts(['id', 'name', 'code'])
            ->defaultSort('id');

        $this->allowedFilters([
            AllowedFilter::exact('id'),
            AllowedFilter::exact('code'),
            AllowedFilter::exact('is_active'),
        ]);

        $this->allowedIncludes([
            AllowedInclude::callback('sections', function ($query) {
                /** @var HasMany $query */
                $query->getQuery()->with([
                    'children' => function ($childrenQuery) {
                        $childrenQuery->orderBy('id');
                    },
                ]);
            }, 'catalogItems'),
        ]);
    }
}
