<?php

namespace App\Http\ApiV1\Modules\Catalogs\Queries;

use App\Domain\Catalogs\Models\Catalog;
use Ensi\QueryBuilderHelpers\Filters\DateFilter;
use Spatie\QueryBuilder\AllowedFilter;
use Spatie\QueryBuilder\AllowedInclude;
use Spatie\QueryBuilder\QueryBuilder;

class CatalogsQuery extends QueryBuilder
{
    public function __construct()
    {
        parent::__construct(Catalog::query());

        $this->allowedSorts([
            'id',
            'name',
            'code',
            'is_active',
            'created_at',
            'updated_at',
        ]);

        $this->defaultSort('id');

        $this->allowedFilters([
            AllowedFilter::exact('id'),
            AllowedFilter::partial('code'),
            AllowedFilter::partial('name'),
            AllowedFilter::exact('is_active'),

            ...DateFilter::make('created_at')->gte()->lte(),
            ...DateFilter::make('updated_at')->gte()->lte(),
        ]);

        $this->allowedIncludes([
            AllowedInclude::relationship('catalog_items', 'catalogItems'),
        ]);
    }
}
