<?php

namespace App\Http\ApiV1\Modules\Catalogs\Requests;

use App\Domain\Catalogs\Models\Catalog;
use App\Domain\Catalogs\Models\CatalogItem;
use App\Domain\Categories\Models\Category;
use App\Http\ApiV1\Support\Requests\BaseFormRequest;
use Illuminate\Validation\Rule;

class PatchCatalogItemRequest extends BaseFormRequest
{
    public function rules(): array
    {
        $catalogId = $this->route('id');

        return [
            'parent_id' => ['sometimes', 'nullable', 'integer', Rule::exists(CatalogItem::class, 'id')],
            'catalog_id' => ['sometimes', 'integer', Rule::exists(Catalog::class, 'id')],
            'category_id' => ['sometimes', 'nullable', 'integer', Rule::exists(Category::class, 'id'), 'prohibits:url'],
            'is_active' => ['sometimes', 'boolean'],
            'url' => ['sometimes', 'nullable', 'string', 'prohibits:category_id'],
            'code' => [
                'sometimes',
                'nullable',
                'regex:/^[a-z0-9_-]+$/i',
                Rule::unique(CatalogItem::class, 'code')->ignore($catalogId),
            ],
            'name' => ['sometimes', 'string'],
        ];
    }
}
