<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Schema;
use Kalnoy\Nestedset\NestedSet;

return new class () extends Migration {
    /**
     * Run the migrations.
     */
    public function up(): void
    {
        Schema::create('catalogs', function (Blueprint $table) {
            $table->id();
            $table->string('name');
            $table->boolean('is_active')->default(false);
            $table->string('code');
            $table->timestamps(6);
        });

        Schema::create('catalog_items', function (Blueprint $table) {
            $table->id();
            $table->string('code');
            $table->string('name');
            $table->string('url')->nullable();

            $table->unsignedBigInteger(NestedSet::LFT)->default(0);
            $table->unsignedBigInteger(NestedSet::RGT)->default(0);
            $table->unsignedBigInteger(NestedSet::PARENT_ID)->nullable();

            $table->index(NestedSet::getDefaultColumns());

            $table->boolean('is_active')->default(false);
            $table->unsignedBigInteger('catalog_id');
            $table->unsignedBigInteger('category_id')->nullable();

            $table->timestamps(6);

            $table->foreign('catalog_id')->references('id')->on('catalogs');
            $table->foreign('category_id')->references('id')->on('categories');

            $table->foreign(NestedSet::PARENT_ID)->references('id')->on('catalog_items');
        });
    }

    /**
     * Reverse the migrations.
     */
    public function down(): void
    {
        Schema::dropIfExists('catalog_items');
        Schema::dropIfExists('catalogs');
    }
};
